#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "readline.h"

#define UNUSED(VAR) \
    ((void) (VAR))

/**
 * Searches for \p search in each line of \p in and replaces it with
 * stars. The result is written into \p out.
 *
 * \param   search      string to censor
 * \param   in          input file
 * \param   out         output file
 * \return  0 on success, an arbitrary error code on failure
 */
int censor(const char *search, FILE *in, FILE *out)
{
    char* line;
    while ((line = readline(in)) != NULL) {
        char* occurence;
        while ((occurence = strstr(line, search)) != NULL) {
            memset(occurence, '*', strlen(search) * sizeof(char));
        }
        if (fprintf(out, line) < 0) {
            perror("printf error");
            return 1;
        }
        free(line);
    }
    return 1;
}

int main(int argc, char *argv[])
{
    if (argc != 4) {
        fprintf(stderr, "usage: %s STRING SOURCE DESTINATION\n", argv[0]);
        return EXIT_FAILURE;
    }
    char* STRING = argv[1];
    char* INPUT = argv[2];
    char* OUTPUT = argv[3];

    FILE* infp;
    if ((infp = fopen(INPUT, "r")) == NULL) {
        perror("open input");
        return 1;
    }

    FILE* outfp;
    if ((outfp = fopen(OUTPUT, "w")) == NULL) {
        perror("open output");
        return 1;
    }
    censor(STRING, infp, outfp);

    int exit_val = EXIT_SUCCESS;

    if (fclose(infp) != 0) {
        perror("close input");
        exit_val = EXIT_FAILURE;
    }
    if (fclose(outfp) != 0) {
        perror("close output");
        exit_val = EXIT_FAILURE;
    }
    return exit_val;
}
