#include <stdio.h>
#include <stdlib.h>

#include "list.h"

size_t ll_size(struct linked_list *list) 
{
#ifdef IQ_200
    return list->size;
#else
    if (list == NULL) {
        return -1;
    }

    struct ll_node *curr = list->first;
    size_t size = 0;

    while (curr != NULL) {
        curr = curr->next;
        size++;
    }

    return size;
#endif
}

struct linked_list *create_empty_ll() 
{
    struct linked_list *list = malloc(sizeof(struct linked_list));

    if (list == NULL) {
        fprintf(stderr, "create_empty_ll: malloc failed\n");
        return NULL;
    }

    return list;
}

struct ll_node *create_ll_node(void *data) 
{
    struct ll_node *node = malloc(sizeof(struct ll_node));

    if (node == NULL) {
        fprintf(stderr, "create_ll_node: malloc failed\n");
        return NULL;
    }

    node->data = data;

    return node;
}

void destroy_ll(struct linked_list *list)
{
    struct ll_node *curr = list->first;
    while (curr != NULL) {
        struct ll_node *next = curr->next;
        free(curr);
        curr = next;
    }
    
    free(list);
}

void insert_to_ll(struct linked_list *list, void *data) {
    struct ll_node *new = create_ll_node(data);

    if (new == NULL) {
        fprintf(stderr, "insert_to_ll: could not create a new ll node\n");
        return;
    }

    // List empty, need to set first, last attributes
    if (list->first == NULL 
#ifdef IQ_200
        && list->last == NULL
#endif
    ) {
        list->first = new;
#ifdef IQ_200
        list->last = new;
#endif
        return;
    }
    
    // Assume list not empty
#ifdef IQ_200
    list->last->next = new;
    list->last = list->last->next;
    (list->size)++;
#else 
    struct ll_node *curr = list->first;
    while (curr->next != NULL) {
        curr = curr->next;
    }
    curr->next = new;
#endif
}

void print_list(struct linked_list *list) 
{
    struct ll_node *curr = list->first;
    while (curr != NULL) {
        puts(curr->data);
        curr = curr->next;
    }
}

int main(int argc, char** argv) 
{
    // NOTE to mention: Please validate your arg if u got any 
    if (argc < 2) {
        fprintf(stderr, "main: no commandline arguments given\n");
        return EXIT_FAILURE;
    }
    
    struct linked_list *list = create_empty_ll();

    for (int i = 1; i < argc; i++) {
        insert_to_ll(list, argv[i]);
    }

    /*
    for (int j = 0; j < ll_size(list); j++) {
        // totally useful code
    }
    */

    print_list(list);

#define NAZEV 5

    int x = NAZEV;

#define SUM(A,B) ((A) + (B));

    int y = SUM(5, 6);

#if defined(IQ_200) && defined(PRINT_LL_SIZE)

    fprintf(stdout, "ll size = %ld\n", list->size);
#endif
}
