//#define _POSIX_C_SOURCE 200809L
#define _POSIX_C_SOURCE 200112L
#include <sys/types.h>
#include <sys/stat.h>
#include <stdint.h>
#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/sysmacros.h>
#include <unistd.h>
#include <fcntl.h>
#include <dirent.h>

int posix_open(char *path);
int stats(char *path);
int print_dir(char *path);

int main(int argc, char *argv[])
{
    if (argc != 2) {
        fprintf(stderr, "Usage: %s <pathname>\n", argv[0]);
        return EXIT_FAILURE;
    }

    //return posix_open(argv[1]);
    //return stats(argv[1]);
    return print_dir(argv[1]);
}

int posix_open(char *path) {
    // https://linux.die.net/man/2/open
    int fd = open(path, O_WRONLY | O_CREAT | O_TRUNC, 0777);

    if (fd == -1) {
        fprintf(stderr, "open\n");
        return EXIT_FAILURE;
    }

    char buff[] = "Hello World!\n";

    if (write(fd, buff, sizeof(buff)) == -1) {
        fprintf(stderr, "write\n");
        return EXIT_FAILURE;
    }

    if (close(fd) != 0) {
        fprintf(stderr, "close\n");
        return EXIT_FAILURE;
    }

    return EXIT_SUCCESS;
}

int stats(char *path) {
    // https://linux.die.net/man/2/lstat
    struct stat sb;

    if (lstat(path, &sb) == -1) {
        fprintf(stderr, "lstat\n");
        return EXIT_FAILURE;
    }

    printf("ID of containing device:  [%jx,%jx]\n",
            (uintmax_t) major(sb.st_dev),
            (uintmax_t) minor(sb.st_dev));

    printf("File type:                ");

#if (_POSIX_C_SOURCE == 200112L)
    if (S_ISREG(sb.st_mode)) {
        printf("regular file\n");
    }
    if (S_ISDIR(sb.st_mode)) {
        printf("directory\n");
    }
    if (S_ISCHR(sb.st_mode)) {
        printf("character device\n");
    }
    if (S_ISBLK(sb.st_mode)) {
        printf("block device\n");
    }
    if (S_ISFIFO(sb.st_mode)) {
        printf("FIFO/pipe\n");
    }
    if (S_ISLNK(sb.st_mode)) {
        printf("symlink\n");
    }
    if (S_ISSOCK(sb.st_mode)) {
        printf("socket\n");
    }
#endif

#if (_POSIX_C_SOURCE == 200809L)
    switch (sb.st_mode & S_IFMT)
    {
    case S_IFBLK:
        printf("block device\n");
        break;
    case S_IFCHR:
        printf("character device\n");
        break;
    case S_IFDIR:
        printf("directory\n");
        break;
    case S_IFIFO:
        printf("FIFO/pipe\n");
        break;
    case S_IFLNK:
        printf("symlink\n");
        break;
    case S_IFREG:
        printf("regular file\n");
        break;
    case S_IFSOCK:
        printf("socket\n");
        break;
    default:
        printf("unknown?\n");
        break;
    }
#endif

    printf("I-node number:            %ju\n", (uintmax_t) sb.st_ino);
    printf("Mode:                     %jo (octal)\n", (uintmax_t) sb.st_mode);
    printf("Link count:               %ju\n", (uintmax_t) sb.st_nlink);
    printf("Ownership:                UID=%ju   GID=%ju\n", (uintmax_t) sb.st_uid, (uintmax_t) sb.st_gid);
    printf("Preferred I/O block size: %jd bytes\n", (intmax_t) sb.st_blksize);
    printf("File size:                %jd bytes\n", (intmax_t) sb.st_size);
    printf("Blocks allocated:         %jd\n", (intmax_t) sb.st_blocks);
    printf("Last status change:       %s", ctime(&sb.st_ctime));
    printf("Last file access:         %s", ctime(&sb.st_atime));
    printf("Last file modification:   %s", ctime(&sb.st_mtime));

    return EXIT_SUCCESS;
}

int print_dir(char *path) {
    // https://linux.die.net/man/3/opendir
    // https://linux.die.net/man/3/readdir
    DIR *dir = NULL;
    if ((dir = opendir(path)) == NULL) {
        fprintf(stderr, "opendir\n");
        return EXIT_FAILURE;
    }
    struct dirent *dir_entry = NULL;
    while ((dir_entry = readdir(dir)) != NULL) {
        puts(dir_entry->d_name);
    }
    if (closedir(dir) == -1) {
        fprintf(stderr, "closedir\n");
        return EXIT_FAILURE;
    }
    return EXIT_SUCCESS;
}
