#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <stdbool.h>
#include <string.h>
#include <ctype.h>
#include <assert.h>

struct node
{
    struct node* next;
    char* name;
    char* surname;
    char* code;
    char* marks;
    int sum;
    bool failed;
};


struct student_list
{
    struct node* head;
    struct node* tail;
    size_t count;
};


int is_file_valid(char* file)
{
    return access(file, R_OK);
}


 const int file_end = 1;
 const int err = -1;

 int read_line(FILE* file, char* line)
 {
     size_t line_size = 128;
     line = malloc(sizeof(char) * line_size);
     if (line == NULL)
     {
         perror("malloc line1");
         return err;
     }

     int chars_read;
     if ((chars_read = fgets(line, line_size, file)) == NULL)
     {
         return file_end;
     }
     return chars_read;
 }


 int get_newline(FILE* file, char *buf)
 {
     size_t line_size = 128;
     char* buf = malloc(sizeof(char) * line_size);
     if (buf == NULL)
     {
         perror("malloc line1");
         return NULL;
     }
     memset(buf, 0, line_size);
     int result;
     if ((result = read_line(file, buf)) == err)
     {
         return err;
     }
     else if (result == file_end)
     {
         return file_end;
     }
     return result;
 }


 char* get_part(char* data, int len, int *pos, bool (*predicate)(char))
 {
     char* name = malloc(sizeof(char)* 64);
     if (name == NULL)
     {
         perror("malloc name");
         return NULL;
     }
     for (; data[*pos] != ';' && *pos < len; *pos++)
     {
         if (!predicate(data[*pos]))
         {
             free(name);
             return NULL;
         }
         name[*pos] = data[*pos];
     }
     return name;
 }


 bool is_name (char chhr)
 {
     return isalpha(chhr);
 }

 bool is_good(char chhr)
 {
     return chhr == 'A' || chhr == 'B' || chhr == 'C' || chhr == 'D';
 }
 bool is_mark(char chhr)
 {
     return is_good(chhr) || chhr == 'F';
 }

 bool is_anything(char chhr)
 {
     return true;
 }


 bool are_marks_correct(char* marks, int count)
 {
     if (count > 3)
     {
         return err;
     }
     int good = 0;
     int f = 0;
     for (int i = 0; i < count; i++)
     {
         if (marks[i] == 'F')
         {
             f++;
             if (f == 3)
             {
                 return err;
             }
         }
         if (is_good(marks[i]))
         {
             if (good != 0)
             {
                 return err;
             }
             good++;
         }

     }// TODO more checks
 }
 int make_new_entry(char* data, int len, struct node* node)
 {
     int pos = 0;
     char* name = get_part(data, len, &pos, is_name);
     if (name == NULL)
     {
         return err;
     }
     char* surname = get_part(data, len, &pos, is_name);
     if (surname == NULL)
     {
         free(name);
         return err;
     }
     int buf = pos;
     char* code = get_part(data, len, &pos, is_anything);
     if (code == NULL || pos - buf != 5)
     {
         free(name);
         free(surname);
         return err;
     }
     int buf = pos;
     char* marks = get_part(data, len, &pos, is_mark);
     if (marks == NULL)
     {
         free(code);
         free(name);
         free(surname);
         return err;
     }
     buf = pos - buf;
     if (!are_marks_correct(marks, buf))
     {
         free(code);
         free(name);
         free(surname);
         return err;
     }
     node = malloc(sizeof(struct node));
     if (node == NULL)
     {
         free(code);
         free(name);
         free(surname);
         return err;
     }
     memset(node, 0, sizeof(struct node));
     node->name = name;
     node->surname = surname;
     node->code = code;
     node->marks = marks;
     return 0;

 }


 void destroy_node(struct node* node)
 {
     free(node->name);
     free(node->surname);
     free(node->code);
     free(node->marks);
     free(node);
 }


 void destroy_list(struct student_list* list)
 {
     assert(list != NULL);
     struct node* node = list->head;
     while (node != NULL)
     {
         struct node* buf = node->next;
         destroy_node(node);
         node = buf;
     }
     
 }

 int fill_entries(FILE* file,struct student_list *list)
 {
     char* data;
     int len;
     if ((len = get_newline(file, data)) == err)
     {
         return err;
     }
     else if (len == file_end)
     {
         return file_end;
     }

     while (true)
     {
         char* data;
         int len;
         if ((len = get_newline(file, data)) == err)
         {
             return err;
         }
         else if (len == file_end)
         {
             return file_end;
         }
         struct node* node;
         if (make_new_entry(data, len, node) == err)
         {
             free(data);
             destroy_list(list);
             return err;
         }
         if (list->head == NULL)
         {
             list->head = node;
         }
         else
         {
             list->tail->next = node;
             list->tail = node;
         }
         (list->count)++;
     }
     return 0;
 }


 int change_marks(struct node* node)
 {
     char* name = node->name;
     for (int i = 0; name[i] != '\0';i++)
     {
         name[i] = toupper(name[i]);
     }
     if (strcmp(name, "PETR") == 0)
     {
         
     }
 }


 int change_entries(struct student_list* list)
 {
     struct node* node = list->head;
     while (node != NULL)
     {
         change_marks(node);
     }
 }
int main(int argc, char** argv)
{
    if (argc != 2)
    {
        fprintf(stderr, "ivalid input");
        return EXIT_FAILURE;
    }

    char* file_name = argv[1];
    if (is_file_valid(file_name) == -1)
    {
        fprintf(stderr, "invalid filename");
        return EXIT_FAILURE;
    }
    FILE* file = fopen(file_name, "r");
    if (file == NULL)
    {
        fprintf(stderr, "fopen file1");
        return EXIT_FAILURE;
    }
    struct student_list* list = malloc(sizeof(struct student_list));
    if (list == NULL)
    {
        fclose(file);
        return err;
    }
    if (fill_entries(file, list) == err)
    {
        return EXIT_FAILURE;
    }


}

