#include <stdio.h>
#include <string.h>
#include <stdlib.h>

struct student_node {
    char* name;
    char* surname;
    char* course_id;
    char grades[3];
    struct student_node* next;
};

struct linked_list {
    struct student_node* first;
    struct student_node* last;
    size_t size;
};

int init_student(struct student_node* student) {
    student->grades[0] = '-';
    student->grades[1] = '-';
    student->grades[2] = '-';
    student->name = calloc(32, sizeof(char));
    student->surname = calloc(40, sizeof(char));
    student->course_id = calloc(10, sizeof(char));

    if (student->name == NULL || student->surname == NULL || student->course_id == NULL) {
        return 1;
    }
    return 0;
}

void destroy_student(struct student_node* student) {
    free(student->name);
    free(student->surname);
    free(student->course_id);
    free(student);
}

void destroy_linked_list(struct linked_list* list) {
    int size = list->size;
    struct student_node* node = list->first;
    struct student_node* temp;

    while (node != NULL) {
        temp = node->next;
        destroy_student(node);
        node = temp;
    }

}

int get_student(char* text, struct student_node* student) {
    char* temp = calloc(3, sizeof(char));

    if (temp == NULL) { return 1; }

    int scan_size = sscanf(text, "%s ; %s ; %s ; %s ", student->name, student->surname, student->course_id, temp);
    if (scan_size < 0) { free(temp); return 1; }
    for (int i = 0; i < strlen(temp); i++) {
        if (temp[i] < 'A' || temp[i] > 'F') { free(temp); return 1; }
        student->grades[i] = temp[i];
    }

    free(temp);
    return 0;
}

void add_to_linked_list(struct linked_list *list, struct student_node* student) {
    if (list->first == NULL) {
        list->first = student;
    } else {
        list->last->next = student;
    }
    list->last = student;
}

int get_students(struct linked_list *list, FILE* student_file) {
    char c;
    char buffer[2000];
    if (fread(buffer, sizeof(char), 1999, student_file) < 0) { return 1; }

    int beginning = 0;
    for (int i = 0; i < strlen(buffer); i++) {
        if (buffer[i] == '\n') {
            char buffer2[100];
            strncpy(buffer, buffer+beginning, i - beginning);
            struct student_node* student = malloc(sizeof(struct student_node));
            if (init_student(student)) {
                destroy_linked_list(list);
                fprintf(stderr, "error allocating student node");
                return 1;
            }
            if (get_student(buffer2, student)) {
                destroy_linked_list(list);
                fprintf(stderr, "error extracting student information");
                return 1;
            }
            add_to_linked_list(list, student);
        }
    }
    return 0;
}

int main(int argc, char* argv[]) {
    if (argc < 2) {
        fprintf(stderr, "missing input file");
        return EXIT_FAILURE;
    }

    FILE* s_file;
    if ((s_file = fopen(argv[1], "r")) == NULL) {
        fprintf(stderr, "could not open input file");
        return EXIT_FAILURE;
    }

    struct linked_list students[1];
    if (get_students(students, s_file)) {
        return 1;
    }


    return 0;
}
